@@ -352,7 +352,7 @@ VLCMediaDelegate>
 
     _currentAspectRatio = VLCAspectRatioDefault;
     _mediaPlayer.videoAspectRatio = NULL;
-    _mediaPlayer.scaleFactor = 0;
+    _mediaPlayer.videoCropGeometry = NULL;
 
     [self subscribeRemoteCommands];
 
@@ -688,7 +688,7 @@ VLCMediaDelegate>
     NSInteger mediaListCount = _mediaList.count;
 
 #if TARGET_OS_IOS
-    if (mediaListCount > 2 && _shuffleMode) {
+    if (self.repeatMode != VLCRepeatCurrentItem && mediaListCount > 2 && _shuffleMode) {
 
         NSNumber *nextIndex;
         NSUInteger currentIndex = [_mediaList indexOfMedia:_listPlayer.mediaPlayer.media];
@@ -733,46 +733,38 @@ VLCMediaDelegate>
     }
 }
 
+- (NSString *)screenAspectRatio
+{
+    UIScreen *screen = [[UIDevice currentDevice] VLCHasExternalDisplay] ? [UIScreen screens][1] : [UIScreen mainScreen];
+    return [NSString stringWithFormat:@"%d:%d", (int)screen.bounds.size.width, (int)screen.bounds.size.height];
+}
+
+- (void)switchIPhoneXFullScreen
+{
+    BOOL inFullScreen = _mediaPlayer.videoCropGeometry && [[NSString stringWithUTF8String:_mediaPlayer.videoCropGeometry] isEqualToString:[self screenAspectRatio]];
+    if (inFullScreen) {
+        const char *previousAspectRatio = _currentAspectRatio == VLCAspectRatioDefault ? NULL : [[self stringForAspectRatio:_currentAspectRatio] UTF8String];
+        _mediaPlayer.videoAspectRatio = (char *)previousAspectRatio;
+    }
+    _mediaPlayer.videoCropGeometry = inFullScreen ? NULL : (char *)[[self screenAspectRatio] UTF8String];
+}
+
 - (void)switchAspectRatio
 {
-    if (_currentAspectRatio == VLCAspectRatioSixteenToTen) {
-        _mediaPlayer.videoAspectRatio = NULL;
-        _mediaPlayer.scaleFactor = 0;
-        _currentAspectRatio = VLCAspectRatioDefault;
-    } else {
-        _currentAspectRatio++;
-
-        if (_currentAspectRatio == VLCAspectRatioFillToScreen) {
-            UIScreen *screen;
-            if (![[UIDevice currentDevice] VLCHasExternalDisplay])
-                screen = [UIScreen mainScreen];
-            else
-                screen = [UIScreen screens][1];
-
-            float f_ar = screen.bounds.size.width / screen.bounds.size.height;
-
-            if (f_ar == (float)(4.0/3.0) ||
-                f_ar == (float)(1366./1024.)) {
-                // all iPads
-                _mediaPlayer.videoCropGeometry = "4:3";
-            } else if (f_ar == (float)(2./3.) || f_ar == (float)(480./320.)) {
-                // all other iPhones
-                _mediaPlayer.videoCropGeometry = "16:10"; // libvlc doesn't support 2:3 crop
-            } else if (f_ar == .5625) {
-                // AirPlay
-                _mediaPlayer.videoCropGeometry = "16:9";
-            } else if (f_ar == (float)(640./1136.) ||
-                       f_ar == (float)(568./320.) ||
-                       f_ar == (float)(667./375.) ||
-                       f_ar == (float)(736./414.)) {
-                // iPhone 5 and 6 and 6+
-                _mediaPlayer.videoCropGeometry = "16:9";
-            } else
-                APLog(@"unknown screen format %f, can't crop", f_ar);
-        } else {
-            _mediaPlayer.videoAspectRatio = (char *)[[self stringForAspectRatio:_currentAspectRatio] UTF8String];
+    _currentAspectRatio = _currentAspectRatio == VLCAspectRatioSixteenToTen ? VLCAspectRatioDefault : _currentAspectRatio + 1;
+    switch (_currentAspectRatio) {
+        case VLCAspectRatioDefault:
+            _mediaPlayer.videoAspectRatio = NULL;
             _mediaPlayer.videoCropGeometry = NULL;
-        }
+            break;
+        case VLCAspectRatioFillToScreen:
+            _mediaPlayer.videoCropGeometry = (char *)[[self screenAspectRatio] UTF8String];
+            break;
+        case VLCAspectRatioFourToThree:
+        case VLCAspectRatioSixteenToTen:
+        case VLCAspectRatioSixteenToNine:
+            _mediaPlayer.videoCropGeometry = NULL;
+            _mediaPlayer.videoAspectRatio = (char *)[[self stringForAspectRatio:_currentAspectRatio] UTF8String];
     }
 
     if ([self.delegate respondsToSelector:@selector(showStatusMessage:forPlaybackController:)]) {
@@ -976,7 +968,7 @@ VLCMediaDelegate>
 {
     if (_needsMetadataUpdate == NO) {
         _needsMetadataUpdate = YES;
-        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
+        dispatch_async(dispatch_get_main_queue(), ^{
             [self _updateDisplayedMetadata];
         });
     }
